#include "loaderthread.h"
#include "rbloader.h"
#include "globals.h"

#include <wx/url.h>

const wxEventType wxEVT_LOADERTHREAD_ENDED = wxNewEventType();
const wxEventType wxEVT_LOADERTHREAD_ERROR = wxNewEventType();
const wxEventType wxEVT_LOADERTHREAD_CANCEL = wxNewEventType();
const wxEventType wxEVT_LOADERTHREAD_STATE = wxNewEventType();

LoaderThread::LoaderThread(RBLoader* parent,int id) : wxThread()
{
    prntctrl=parent;
    pgb=NULL;
    txtinfos=NULL;
    currsong=NULL;
    bStopWanted=NULL;
    Create();
}

LoaderThread::~LoaderThread()
{
    //dtor
}

void LoaderThread::SetAssociatedControls(wxGauge* gauge,wxStaticText* label)
{
    pgb=gauge;
    txtinfos=label;
}

void LoaderThread::SetSongVar(RB_Song *song)
{
    currsong=song;
}

void LoaderThread::StopRunning()
{
    bStopWanted=true;
}

void* LoaderThread::Entry()
{
    if (currsong==NULL)
    {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_ERROR,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }
    bStopWanted=false;
    if (pgb!=NULL)
    {
        pgb->SetValue(0);
        pgb->SetRange(100);
    }
    if (txtinfos!=NULL)
    {
        txtinfos->SetLabel(_T("Recherche du fichier : \351tape 1/2."));
    }
    // On charge la page d'ouverture du fichier
    wxString sBase=_T("http://radioblogclub.com/open/");
    wxString pageUrl=sBase + currsong->Url;
    wxString sPage=LoadWebPage(pageUrl,bStopWanted);
    if (sPage==_T("_LWP_ERROR_")) // Y a t'il eut une erreur lors du chargement ?
    {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_ERROR,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }
    if (sPage==_T("_LWP_CANCELLED_")) // Y a t'il eut annulation lors du chargement ?
    {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_CANCEL,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }
    // On recherche l'adresse exacte de la source
    int i=sPage.Find(_T("javascript:openRadio('"))+22;
    int j=sPage.Find(_T("?autoplay=')"));
    wxString RB_Url=sPage.Mid(i,j-i);
    // On charge la PlayList distante
    if (txtinfos!=NULL)
    {
        txtinfos->SetLabel(_T("Recherche du fichier : \351tape 2/2."));
    }
    wxString sPlayList=LoadWebPage(RB_Url + _T("sounds/playlist.php"),bStopWanted);
    if (sPage==_T("_LWP_ERROR_")) // Y a t'il eut une erreur lors du chargement ?
    {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_ERROR,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }
    if (sPage==_T("_LWP_CANCELLED_")) // Y a t'il eut annulation lors du chargement ?
    {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_CANCEL,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }
    // Recherche du fichier dans la PlayList
    //i=sPlayList.Find(_T("<track "));
    j=sPlayList.Find(_T("title=\""));
    wxString sPath=_T(""),sName=_T(""),sTitle=_T("");
    int l=0;
    while(j!=-1)
    {
        j+=7;
        sPlayList.Remove(0,j);
        i=sPlayList.Find(_T("\""))-1;
        sTitle=sPlayList.Left(i+1);
        i=sPlayList.Find(_T("path=\""));
        l++;
        if (i!=-1)
        {
            j=sPlayList.Find(_T("\"/>"));
            sPath=sPlayList.Mid(i+6,j-i-6);
            sPlayList.Remove(0,j+1);
            if ((sTitle==currsong->Name)||(sPath.Find(currsong->Name)!=-1)) break;
            sPath=_T("");
            sTitle=_T("");
        } else {
            //wxMessageBox(_T("Erreur lors de la lecture de la playlist..."),_T("Erreur"),wxICON_EXCLAMATION);
            wxCommandEvent evt(wxEVT_LOADERTHREAD_ERROR,prntctrl->GetId());
            prntctrl->GetEventHandler()->AddPendingEvent(evt);
            return NULL;
        }
        j=sPlayList.Find(_T("title=\""));
    }
    if (!sPath.IsEmpty())
    {
        if (sPath.Find(_T("http://"))!=-1)
            currsong->Url=sPath;
        else
            currsong->Url=RB_Url+sPath;
    } else {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_ERROR,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }
    // Si on arrive ici, c'est qu'on a trouv l'url exacte du fichier  tlcharger
    // On peut donc envoyer l'vnement de changement d'tat
    wxCommandEvent evt1(wxEVT_LOADERTHREAD_STATE,prntctrl->GetId());
    prntctrl->GetEventHandler()->AddPendingEvent(evt1);
    if (txtinfos!=NULL)
    {
        txtinfos->SetLabel(_T("T\351l\351chargement du fichier."));
    }
    wxURL url(currsong->Url);
    if ((!url.IsOk())&&(url.GetError()!=wxURL_NOERR))
    {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_ERROR,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }

    wxInputStream* inputstream=url.GetInputStream();
    if (inputstream==NULL)
    {
        wxCommandEvent evt(wxEVT_LOADERTHREAD_ERROR,prntctrl->GetId());
        prntctrl->GetEventHandler()->AddPendingEvent(evt);
        return NULL;
    }

    size_t flen=inputstream->GetSize();
    size_t done=0;
    const int iBuffSize=256;
    char szBuff[iBuffSize+2];
    size_t rd=1;
    int prct;
    wxStopWatch tmr;
    long millis;
    float spd;
    size_t done2=0;
    wxString slabel;
    while(rd>0)
    {
        inputstream->Read(szBuff,iBuffSize);
        rd=inputstream->LastRead();
        if (rd>0)
        {
            done+=rd;
            done2+=rd;
            currsong->Datas.AppendData(szBuff,rd);
            if (flen>0)
            {
                prct=(int)((100*done)/flen);
                if (pgb!=NULL) pgb->SetValue(prct);
            }
            millis=tmr.Time();
            if (millis>1000)
            {
                spd=(done2*1000.)/(float)millis;
                done2=0;
                slabel=_T("T\351l\351chargement \340 ");
                tmr.Start();
                if (spd<1024)
                {
                    slabel.Append(wxString::Format(_T("%d octets/sec."),(int)spd));
                } else {
                    spd/=1024.;
                    if (spd<1024.)
                    {
                        slabel.Append(wxString::Format(_T("%0.2f Ko/sec."),spd));
                    } else {
                        spd/=1024.;
                        slabel.Append(wxString::Format(_T("%0.2f Mo/sec."),spd));
                    }
                }
                if (txtinfos!=NULL) txtinfos->SetLabel(slabel);
            }
        } else {
            break;
        }
        if ((TestDestroy())||(bStopWanted))
        {
                delete inputstream;
                wxCommandEvent evt(wxEVT_LOADERTHREAD_CANCEL,prntctrl->GetId());
                prntctrl->GetEventHandler()->AddPendingEvent(evt);
                return NULL;
        }
    }
    delete inputstream;
    // Le travail est termin
    currsong->State=2;
    // Envoyer l'vnement de fin de thread au contrle parent
    wxCommandEvent evt2(wxEVT_LOADERTHREAD_ENDED,prntctrl->GetId());
    prntctrl->GetEventHandler()->AddPendingEvent(evt2);

    // C'est termin pour le thread
    return (void*)1;

}
