#include "rbloader.h"

BEGIN_EVENT_TABLE(RBLoader,wxPanel)
    EVT_LOADERTHREAD_ENDED(-1,RBLoader::OnThreadEnded)
    EVT_LOADERTHREAD_ERROR(-1,RBLoader::OnThreadError)
    EVT_LOADERTHREAD_CANCEL(-1,RBLoader::OnThreadCancelled)
    EVT_BUTTON(wxID_CANCEL,RBLoader::OnCancelButtonClicked)
    EVT_LOADERTHREAD_STATE(-1,RBLoader::OnThreadStateChanged)
END_EVENT_TABLE()

RBLoader::RBLoader(wxWindow* parent,wxWindowID id,const wxString& title) : wxPanel(parent,id)
{
    wxStaticBoxSizer* mainsizer=new wxStaticBoxSizer(wxVERTICAL,this,title);
    stbMain=mainsizer->GetStaticBox();
        lblCurFile=new wxStaticText(this,-1,_T(""),wxDefaultPosition,wxDefaultSize,wxST_NO_AUTORESIZE);
        mainsizer->Add(lblCurFile,0,wxLEFT|wxRIGHT|wxEXPAND,5);
        pgbLdState=new wxGauge(this,-1,100,wxDefaultPosition,wxSize(-1,15));
        pgbLdState->SetValue(0);
        mainsizer->Add(pgbLdState,0,wxLEFT|wxRIGHT|wxTOP|wxEXPAND,5);
        wxBoxSizer *sizer2=new wxBoxSizer(wxHORIZONTAL);
            lblInfos=new wxStaticText(this,-1,_T(""),wxDefaultPosition,wxDefaultSize,wxST_NO_AUTORESIZE);
            sizer2->Add(lblInfos,1,wxALL|wxCENTER,0);
            btnCancel=new wxButton(this,wxID_CANCEL,_T("Annuler"));
            btnCancel->Enable(false);
            sizer2->Add(btnCancel,0,wxLEFT|wxEXPAND,5);
        mainsizer->Add(sizer2,0,wxALL|wxEXPAND,5);
    SetSizer(mainsizer);
    // Initialisations des variables
    ldrthrd=NULL;
    currsong=NULL;
    // Cration et connection du Timer
    int ID_Timer=wxNewId();
    animtimer=new wxTimer(this,ID_Timer);
    Connect(ID_Timer,wxEVT_TIMER,wxTimerEventHandler(RBLoader::OnTimer));
}

RBLoader::~RBLoader()
{
    // Dtruire le Timer
    delete animtimer;
    //Dtruire l'ventuel Thread de chargement
}

bool RBLoader::IsWorking()
{
    return (ldrthrd!=NULL);
}

void RBLoader::OnCancelButtonClicked(wxCommandEvent &event)
{
    if (ldrthrd==NULL) return;
    ldrthrd->Pause();
    int iRes=wxMessageBox(_T("Arr\352ter la r\351cup\351ration du fichier :\n")+currsong->Name + _T(" ?"),_T("Annuler"),wxICON_QUESTION|wxYES_NO);
    ldrthrd->Resume();
    if (iRes==wxYES) ldrthrd->StopRunning();
}

void RBLoader::OnThreadEnded(wxCommandEvent &event)
{
    // Si on reoit cet vnement, c'est que le thread est termin
    ldrthrd=NULL;
    // On fait suivre l'vnement  la wxWindow parente
    event.Skip();
    // On dsactive le bouton "Annuler"
    btnCancel->Enable(false);
    // On vrifie que le Timer soit bien arrt
    animtimer->Stop();
    // On remets  zro les contrles d'informations
    lblInfos->SetLabel(_T(""));
    pgbLdState->SetValue(0);
    lblCurFile->SetLabel(_T(""));
}

void RBLoader::OnThreadError(wxCommandEvent &event)
{
    // Si on reoit cet vnement, c'est que le thread est termin
    ldrthrd=NULL;
    // On fait suivre l'vnement  la wxWindow parente
    event.Skip();
    // On vrifie que le Timer soit bien arrt
    animtimer->Stop();
    // On dsactive le bouton "Annuler"
    btnCancel->Enable(false);
    // On remets  zro les contrles d'informations
    lblInfos->SetLabel(_T(""));
    pgbLdState->SetValue(0);
    lblCurFile->SetLabel(_T(""));
}

void RBLoader::OnThreadCancelled(wxCommandEvent &event)
{
    // Si on reoit cet vnement, c'est que le thread est termin
    ldrthrd=NULL;
    // On fait suivre l'vnement  la wxWindow parente
    event.Skip();
    // On vrifie que le Timer soit bien arrt
    animtimer->Stop();
    // On dsactive le bouton "Annuler"
    btnCancel->Enable(false);
    // On remets  zro les contrles d'informations
    lblInfos->SetLabel(_T(""));
    pgbLdState->SetValue(0);
    lblCurFile->SetLabel(_T(""));
}

bool RBLoader::SetCurrentJob(RB_Song *entry)
{
    // Si le thread existe encore, c'est qu'il travaille
    if (ldrthrd!=NULL) return false;
    // Si le thread ne travaille plus, il faut remettre la variable
    //  NULL avant de lui affecter une autre valeur
    if ((currsong!=NULL)&&(entry!=NULL)) return false;
    // Si on arrive ici, on peut affecter la variable sans risque
    currsong=entry;
    // Si la valeur passe en paramtre est nulle, on sort
    if (currsong==NULL) return true;
    // On marque la variable comme tant en cours de chargement
    currsong->State=1;
    // Sinon, c'est bon, on peut crer le thread de chargement
    ldrthrd=new LoaderThread(this,-1);
    ldrthrd->SetAssociatedControls(pgbLdState,lblInfos);
    ldrthrd->SetSongVar(currsong);
    // Mettre  jour les contrles
    lblCurFile->SetLabel(currsong->Name);
    // Activer le bouton Annuler
    btnCancel->Enable();
    // Mettre en route le thread
    ldrthrd->Run();
    // Mettre en route le Timer pour nimer la progressbar pendant le pre-load
    animtimer->Start(100);
    return true;
}

RB_Song* RBLoader::GetCurrentJobPtr()
{
    return currsong;
}

void RBLoader::OnTimer(wxTimerEvent& event)
{
    if (ldrthrd==NULL) animtimer->Stop();
    pgbLdState->Pulse();
}

void RBLoader::OnThreadStateChanged(wxCommandEvent &event)
{
    // Le thread a fini le pre-load -> On peut arrter le Timer d'animation
    animtimer->Stop();
}
